@@ -1,5 +1,5 @@
 /*
- * i-scream central monitoring system
+ * i-scream libstatgrab
  * http://www.i-scream.org
  * Copyright (C) 2000-2004 i-scream
  *
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: os_info.c,v 1.6 2004/01/19 16:49:19 tdb Exp $
+ * $Id: os_info.c,v 1.8 2004/04/06 14:52:56 tdb Exp $
  */
 
 #include <stdio.h>
@@ -27,18 +27,18 @@
 
 int main(int argc, char **argv){
 
-	general_stat_t *general_stats;
+	sg_host_info *general_stats;
 
 	/* Initialise statgrab */
-	statgrab_init();
+	sg_init();
 
 	/* Drop setuid/setgid privileges. */
-	if (statgrab_drop_privileges() != 0) {
+	if (sg_drop_privileges() != 0) {
 		perror("Error. Failed to drop privileges");
 		return 1;
 	}
 
-	general_stats = get_general_stats();
+	general_stats = sg_get_host_info();
 
 	if(general_stats == NULL){
 		fprintf(stderr, "Failed to get os stats\n");
