@@ -1,5 +1,5 @@
 /*
- * i-scream central monitoring system
+ * i-scream libstatgrab
  * http://www.i-scream.org
  * Copyright (C) 2000-2004 i-scream
  * 
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: network_stats.c,v 1.56 2004/03/09 12:43:41 ats Exp $
+ * $Id: network_stats.c,v 1.68 2004/04/08 13:44:44 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include "statgrab.h"
+#include "vector.h"
+#include "tools.h"
 #include <time.h>
 #ifdef SOLARIS
 #include <kstat.h>
@@ -38,16 +40,15 @@
 #include <net/if.h>
 #include <netinet/in.h>
 #include <sys/sockio.h>
+#include <unistd.h>
 #endif
 #ifdef LINUX
 #include <stdio.h>
 #include <sys/types.h>
-#include <regex.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <net/if.h>
 #include <ctype.h>
-#include "tools.h"
 /* Stuff which could be defined by defining KERNEL, but 
  * that would be a bad idea, so we'll just declare it here
  */
@@ -66,61 +67,34 @@ typedef __uint64_t u64;
 #include <net/if.h>
 #include <net/if_media.h>
 #include <sys/ioctl.h>
+#include <unistd.h>
 #endif
 
-static network_stat_t *network_stats=NULL;
-static int interfaces=0;
-
-void network_stat_init(int start, int end, network_stat_t *net_stats){
-
-	for(net_stats+=start; start<end; start++){
-		net_stats->interface_name=NULL;
-		net_stats->tx=0;
-		net_stats->rx=0;
-		net_stats->ipackets=0;
-		net_stats->opackets=0;
-		net_stats->ierrors=0;
-		net_stats->oerrors=0;
-		net_stats->collisions=0;
-		net_stats++;
-	}
+static void network_stat_init(sg_network_io_stats *s) {
+	s->interface_name = NULL;
+	s->tx = 0;
+	s->rx = 0;
+	s->ipackets = 0;
+	s->opackets = 0;
+	s->ierrors = 0;
+	s->oerrors = 0;
+	s->collisions = 0;
 }
 
-network_stat_t *network_stat_malloc(int needed_entries, int *cur_entries, network_stat_t *net_stats){
-
-	if(net_stats==NULL){
-
-		if((net_stats=malloc(needed_entries * sizeof(network_stat_t)))==NULL){
-			return NULL;
-		}
-		network_stat_init(0, needed_entries, net_stats);
-		*cur_entries=needed_entries;
-
-		return net_stats;
-	}
-
-
-	if(*cur_entries<needed_entries){
-		net_stats=realloc(net_stats, (sizeof(network_stat_t)*needed_entries));
-		if(net_stats==NULL){
-			return NULL;
-		}
-		network_stat_init(*cur_entries, needed_entries, net_stats);
-		*cur_entries=needed_entries;
-	}
-
-	return net_stats;
+static void network_stat_destroy(sg_network_io_stats *s) {
+	free(s->interface_name);
 }
 
+VECTOR_DECLARE_STATIC(network_stats, sg_network_io_stats, 5,
+		      network_stat_init, network_stat_destroy);
 
-network_stat_t *get_network_stats(int *entries){
-
-	static int sizeof_network_stats=0;	
-	network_stat_t *network_stat_ptr;
+sg_network_io_stats *sg_get_network_io_stats(int *entries){
+	int interfaces;
+	sg_network_io_stats *network_stat_ptr;
 
 #ifdef SOLARIS
-        kstat_ctl_t *kc;
-        kstat_t *ksp;
+	kstat_ctl_t *kc;
+	kstat_t *ksp;
 	kstat_named_t *knp;
 #endif
 
@@ -138,6 +112,7 @@ network_stat_t *get_network_stats(int *entries){
 
 #ifdef ALLBSD
 	if(getifaddrs(&net) != 0){
+		sg_set_error(SG_ERROR_GETIFADDRS, NULL);
 		return NULL;
 	}
 
@@ -145,15 +120,16 @@ network_stat_t *get_network_stats(int *entries){
 	
 	for(net_ptr=net;net_ptr!=NULL;net_ptr=net_ptr->ifa_next){
 		if(net_ptr->ifa_addr->sa_family != AF_LINK) continue;
-		network_stats=network_stat_malloc((interfaces+1), &sizeof_network_stats, network_stats);
-		if(network_stats==NULL){
+
+		if (VECTOR_RESIZE(network_stats, interfaces + 1) < 0) {
 			return NULL;
 		}
 		network_stat_ptr=network_stats+interfaces;
 		
-		if(network_stat_ptr->interface_name!=NULL) free(network_stat_ptr->interface_name);
-		network_stat_ptr->interface_name=strdup(net_ptr->ifa_name);
-		if(network_stat_ptr->interface_name==NULL) return NULL;
+		if (sg_update_string(&network_stat_ptr->interface_name,
+				     net_ptr->ifa_name) < 0) {
+			return NULL;
+		}
 		net_data=(struct if_data *)net_ptr->ifa_data;
 		network_stat_ptr->rx=net_data->ifi_ibytes;
 		network_stat_ptr->tx=net_data->ifi_obytes;
@@ -169,15 +145,16 @@ network_stat_t *get_network_stats(int *entries){
 #endif
 
 #ifdef SOLARIS
-        if ((kc = kstat_open()) == NULL) {
-                return NULL;
-        }
+	if ((kc = kstat_open()) == NULL) {
+		sg_set_error(SG_ERROR_KSTAT_OPEN, NULL);
+		return NULL;
+	}
 
 	interfaces=0;
 
-    	for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
-        	if (!strcmp(ksp->ks_class, "net")) {
-                	kstat_read(kc, ksp, NULL);
+	for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
+		if (!strcmp(ksp->ks_class, "net")) {
+			kstat_read(kc, ksp, NULL);
 
 #ifdef SOL7
 #define LRX "rbytes"
@@ -205,8 +182,7 @@ network_stat_t *get_network_stats(int *entries){
 			}
 
 			/* Create new network_stats */
-			network_stats=network_stat_malloc((interfaces+1), &sizeof_network_stats, network_stats);
-			if(network_stats==NULL){
+			if (VECTOR_RESIZE(network_stats, interfaces + 1) < 0) {
 				return NULL;
 			}
 			network_stat_ptr=network_stats+interfaces;
@@ -251,10 +227,10 @@ network_stat_t *get_network_stats(int *entries){
 			network_stat_ptr->collisions=knp->value.ui32;
 
 			/* Read interface name */
-			if(network_stat_ptr->interface_name!=NULL){
-				free(network_stat_ptr->interface_name);
+			if (sg_update_string(&network_stat_ptr->interface_name,
+					     ksp->ks_name) < 0) {
+				return NULL;
 			}
-			network_stat_ptr->interface_name=strdup(ksp->ks_name);
 
 			/* Store systime */
 			network_stat_ptr->systime=time(NULL);
@@ -268,6 +244,7 @@ network_stat_t *get_network_stats(int *entries){
 #ifdef LINUX
 	f=fopen("/proc/net/dev", "r");
 	if(f==NULL){
+		sg_set_error(SG_ERROR_OPEN, "/proc/net/dev");
 		return NULL;
 	}
 	/* read the 2 lines.. Its the title, so we dont care :) */
@@ -276,6 +253,7 @@ network_stat_t *get_network_stats(int *entries){
 
 
 	if((regcomp(&regex, "^ *([^:]+): *([0-9]+) +([0-9]+) +([0-9]+) +[0-9]+ +[0-9]+ +[0-9]+ +[0-9]+ +[0-9]+ +([0-9]+) +([0-9]+) +([0-9]+) +[0-9]+ +[0-9]+ +([0-9]+)", REG_EXTENDED))!=0){
+		sg_set_error(SG_ERROR_PARSE, NULL);
 		return NULL;
 	}
 
@@ -285,24 +263,24 @@ network_stat_t *get_network_stats(int *entries){
 		if((regexec(&regex, line, 9, line_match, 0))!=0){
 			continue;
 		}
-        	network_stats=network_stat_malloc((interfaces+1), &sizeof_network_stats, network_stats);
-       		if(network_stats==NULL){
-      			return NULL;
+
+		if (VECTOR_RESIZE(network_stats, interfaces + 1) < 0) {
+			return NULL;
 		}
-	        network_stat_ptr=network_stats+interfaces;
+		network_stat_ptr=network_stats+interfaces;
 
 		if(network_stat_ptr->interface_name!=NULL){
 			free(network_stat_ptr->interface_name);
 		}
 
-		network_stat_ptr->interface_name=get_string_match(line, &line_match[1]);
-		network_stat_ptr->rx=get_ll_match(line, &line_match[2]);
-		network_stat_ptr->tx=get_ll_match(line, &line_match[5]);
-		network_stat_ptr->ipackets=get_ll_match(line, &line_match[3]);
-		network_stat_ptr->opackets=get_ll_match(line, &line_match[6]);
-		network_stat_ptr->ierrors=get_ll_match(line, &line_match[4]);
-		network_stat_ptr->oerrors=get_ll_match(line, &line_match[7]);
-		network_stat_ptr->collisions=get_ll_match(line, &line_match[8]);
+		network_stat_ptr->interface_name=sg_get_string_match(line, &line_match[1]);
+		network_stat_ptr->rx=sg_get_ll_match(line, &line_match[2]);
+		network_stat_ptr->tx=sg_get_ll_match(line, &line_match[5]);
+		network_stat_ptr->ipackets=sg_get_ll_match(line, &line_match[3]);
+		network_stat_ptr->opackets=sg_get_ll_match(line, &line_match[6]);
+		network_stat_ptr->ierrors=sg_get_ll_match(line, &line_match[4]);
+		network_stat_ptr->oerrors=sg_get_ll_match(line, &line_match[7]);
+		network_stat_ptr->collisions=sg_get_ll_match(line, &line_match[8]);
 		network_stat_ptr->systime=time(NULL);
 
 		interfaces++;
@@ -313,6 +291,7 @@ network_stat_t *get_network_stats(int *entries){
 #endif
 
 #ifdef CYGWIN
+	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
 	return NULL;
 #endif
 
@@ -321,7 +300,7 @@ network_stat_t *get_network_stats(int *entries){
 	return network_stats;	
 }
 
-long long transfer_diff(long long new, long long old){
+static long long transfer_diff(long long new, long long old){
 #if defined(SOL7) || defined(LINUX) || defined(FREEBSD) || defined(DFBSD) || defined(OPENBSD)
 	/* 32-bit quantities, so we must explicitly deal with wraparound. */
 #define MAXVAL 0x100000000LL
@@ -336,20 +315,20 @@ long long transfer_diff(long long new, long long old){
 #endif
 }
 
-network_stat_t *get_network_stats_diff(int *entries) {
-	static network_stat_t *diff = NULL;
-	static int diff_count = 0;
-	network_stat_t *src, *dest;
-	int i, j, new_count;
+sg_network_io_stats *sg_get_network_io_stats_diff(int *entries) {
+	VECTOR_DECLARE_STATIC(diff, sg_network_io_stats, 1,
+			      network_stat_init, network_stat_destroy);
+	sg_network_io_stats *src = NULL, *dest;
+	int i, j, diff_count, new_count;
 
 	if (network_stats == NULL) {
 		/* No previous stats, so we can't calculate a difference. */
-		return get_network_stats(entries);
+		return sg_get_network_io_stats(entries);
 	}
 
 	/* Resize the results array to match the previous stats. */
-	diff = network_stat_malloc(interfaces, &diff_count, diff);
-	if (diff == NULL) {
+	diff_count = VECTOR_SIZE(network_stats);
+	if (VECTOR_RESIZE(diff, diff_count) < 0) {
 		return NULL;
 	}
 
@@ -358,10 +337,10 @@ network_stat_t *get_network_stats_diff(int *entries) {
 		src = &network_stats[i];
 		dest = &diff[i];
 
-		if (dest->interface_name != NULL) {
-			free(dest->interface_name);
+		if (sg_update_string(&dest->interface_name,
+				     src->interface_name) < 0) {
+			return NULL;
 		}
-		dest->interface_name = strdup(src->interface_name);
 		dest->rx = src->rx;
 		dest->tx = src->tx;
 		dest->ipackets = src->ipackets;
@@ -373,7 +352,7 @@ network_stat_t *get_network_stats_diff(int *entries) {
 	}
 
 	/* Get a new set of stats. */
-	if (get_network_stats(&new_count) == NULL) {
+	if (sg_get_network_io_stats(&new_count) == NULL) {
 		return NULL;
 	}
 
@@ -410,84 +389,67 @@ network_stat_t *get_network_stats_diff(int *entries) {
 	*entries = diff_count;
 	return diff;
 }
+
+int sg_network_io_compare_name(const void *va, const void *vb) {
+	const sg_network_io_stats *a = (const sg_network_io_stats *)va;
+	const sg_network_io_stats *b = (const sg_network_io_stats *)vb;
+
+	return strcmp(a->interface_name, b->interface_name);
+}
+
 /* NETWORK INTERFACE STATS */
 
-void network_iface_stat_init(int start, int end, network_iface_stat_t *net_stats){
-
-	for(net_stats+=start; start<end; start++){
-		net_stats->interface_name=NULL;
-		net_stats->speed=0;
-		net_stats->dup=UNKNOWN_DUPLEX;
-		net_stats++;
-	}
+static void network_iface_stat_init(sg_network_iface_stats *s) {
+	s->interface_name = NULL;
+	s->speed = 0;
+	s->dup = SG_IFACE_DUPLEX_UNKNOWN;
 }
 
-network_iface_stat_t *network_iface_stat_malloc(int needed_entries, int *cur_entries, network_iface_stat_t *net_stats){
-
-	if(net_stats==NULL){
-
-		if((net_stats=malloc(needed_entries * sizeof(network_iface_stat_t)))==NULL){
-			return NULL;
-		}
-		network_iface_stat_init(0, needed_entries, net_stats);
-		*cur_entries=needed_entries;
-
-		return net_stats;
-	}
-
-
-	if(*cur_entries<needed_entries){
-		net_stats=realloc(net_stats, (sizeof(network_iface_stat_t)*needed_entries));
-		if(net_stats==NULL){
-			return NULL;
-		}
-		network_iface_stat_init(*cur_entries, needed_entries, net_stats);
-		*cur_entries=needed_entries;
-	}
-
-	return net_stats;
+static void network_iface_stat_destroy(sg_network_iface_stats *s) {
+	free(s->interface_name);
 }
 
-network_iface_stat_t *get_network_iface_stats(int *entries){
-	static network_iface_stat_t *network_iface_stats;
-	network_iface_stat_t *network_iface_stat_ptr;
-	static int sizeof_network_iface_stats=0;	
+sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
+	VECTOR_DECLARE_STATIC(network_iface_stats, sg_network_iface_stats, 5,
+			      network_iface_stat_init, network_iface_stat_destroy);
+	sg_network_iface_stats *network_iface_stat_ptr;
 	int ifaces = 0;
 
 #ifdef SOLARIS
-        kstat_ctl_t *kc;
-        kstat_t *ksp;
+	kstat_ctl_t *kc;
+	kstat_t *ksp;
 	kstat_named_t *knp;
 	int sock;
 #endif
 #ifdef ALLBSD
-        struct ifaddrs *net, *net_ptr;
+	struct ifaddrs *net, *net_ptr;
 	struct ifmediareq ifmed;
 	struct ifreq ifr;
 	int sock;
 	int x;
 #endif
 #ifdef LINUX
-        FILE *f;
-        /* Horrible big enough, but it should be easily big enough */
-        char line[8096];
+	FILE *f;
+	/* Horrible big enough, but it should be easily big enough */
+	char line[8096];
 	int sock;
 #endif
 
 #ifdef ALLBSD
-        if(getifaddrs(&net) != 0){
-                return NULL;
-        }
+	if(getifaddrs(&net) != 0){
+		sg_set_error(SG_ERROR_GETIFADDRS, NULL);
+		return NULL;
+	}
 
 	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == 0) return NULL;
 
 	for(net_ptr=net; net_ptr!=NULL; net_ptr=net_ptr->ifa_next){
-                if(net_ptr->ifa_addr->sa_family != AF_LINK) continue;
-                network_iface_stats=network_iface_stat_malloc((ifaces+1), &sizeof_network_iface_stats, network_iface_stats);
-                if(network_iface_stats==NULL){
-                        return NULL;
-                }
-                network_iface_stat_ptr = network_iface_stats + ifaces;
+		if(net_ptr->ifa_addr->sa_family != AF_LINK) continue;
+
+		if (VECTOR_RESIZE(network_iface_stats, ifaces + 1) < 0) {
+			return NULL;
+		}
+		network_iface_stat_ptr = network_iface_stats + ifaces;
 
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, net_ptr->ifa_name, sizeof(ifr.ifr_name));
@@ -501,16 +463,17 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 			network_iface_stat_ptr->up = 0;
 		}
 
-		if (network_iface_stat_ptr->interface_name != NULL) free(network_iface_stat_ptr->interface_name);
-		network_iface_stat_ptr->interface_name = strdup(net_ptr->ifa_name);
-		if (network_iface_stat_ptr->interface_name == NULL) return NULL;
+		if (sg_update_string(&network_iface_stat_ptr->interface_name,
+				     net_ptr->ifa_name) < 0) {
+			return NULL;
+		}
 
 		network_iface_stat_ptr->speed = 0;
-		network_iface_stat_ptr->dup = UNKNOWN_DUPLEX;
+		network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
 		ifaces++;
 
 		memset(&ifmed, 0, sizeof(struct ifmediareq));
-		strlcpy(ifmed.ifm_name, net_ptr->ifa_name, sizeof(ifmed.ifm_name));
+		sg_strlcpy(ifmed.ifm_name, net_ptr->ifa_name, sizeof(ifmed.ifm_name));
 		if(ioctl(sock, SIOCGIFMEDIA, (caddr_t)&ifmed) == -1){
 			/* Not all interfaces support the media ioctls. */
 			continue;
@@ -563,11 +526,11 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 		}
 
 		if( (ifmed.ifm_active | IFM_FDX) == ifmed.ifm_active ){
-			network_iface_stat_ptr->dup = FULL_DUPLEX;
+			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
 		}else if( (ifmed.ifm_active | IFM_HDX) == ifmed.ifm_active ){
-			network_iface_stat_ptr->dup = HALF_DUPLEX;
+			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
 		}else{
-			network_iface_stat_ptr->dup = UNKNOWN_DUPLEX;
+			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
 		}
 
 	}	
@@ -577,10 +540,12 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 
 #ifdef SOLARIS
 	if ((kc = kstat_open()) == NULL) {
+		sg_set_error(SG_ERROR_KSTAT_OPEN, NULL);
 		return NULL;
 	}
 
 	if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
+		sg_set_error(SG_ERROR_SOCKET, NULL);
 		return NULL;
 	}
 
@@ -596,16 +561,16 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 				continue;
 			}
 
-			network_iface_stats = network_iface_stat_malloc(ifaces + 1, &sizeof_network_iface_stats, network_iface_stats);
-			if (network_iface_stats == NULL) {
+			if (VECTOR_RESIZE(network_iface_stats, ifaces + 1) < 0) {
 				return NULL;
 			}
 			network_iface_stat_ptr = network_iface_stats + ifaces;
 			ifaces++;
 
-			if (network_iface_stat_ptr->interface_name != NULL) free(network_iface_stat_ptr->interface_name);
-			network_iface_stat_ptr->interface_name = strdup(ksp->ks_name);
-			if (network_iface_stat_ptr->interface_name == NULL) return NULL;
+			if (sg_update_string(&network_iface_stat_ptr->interface_name,
+					     ksp->ks_name) < 0) {
+				return NULL;
+			}
 
 			if ((ifr.ifr_flags & IFF_UP) != 0) {
 				network_iface_stat_ptr->up = 1;
@@ -619,14 +584,14 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 				network_iface_stat_ptr->speed = 0;
 			}
 
-			network_iface_stat_ptr->dup = UNKNOWN_DUPLEX;
+			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
 			if ((knp = kstat_data_lookup(ksp, "link_duplex")) != NULL) {
 				switch (knp->value.ui32) {
 				case 1:
-					network_iface_stat_ptr->dup = HALF_DUPLEX;
+					network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
 					break;
 				case 2:
-					network_iface_stat_ptr->dup = FULL_DUPLEX;
+					network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
 					break;
 				}
 			}
@@ -638,76 +603,87 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 #endif	
 #ifdef LINUX
 	f = fopen("/proc/net/dev", "r");
-        if(f == NULL){
-                return NULL;
-        }
+	if(f == NULL){
+		sg_set_error(SG_ERROR_OPEN, "/proc/net/dev");
+		return NULL;
+	}
 
 	/* Setup stuff so we can do the ioctl to get the info */
 	if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
+		sg_set_error(SG_ERROR_SOCKET, NULL);
 		return NULL;
 	}
 
 	/* Ignore first 2 lines.. Just headings */
-        if((fgets(line, sizeof(line), f)) == NULL) return NULL;
-        if((fgets(line, sizeof(line), f)) == NULL) return NULL;
+	if((fgets(line, sizeof(line), f)) == NULL) {
+		sg_set_error(SG_ERROR_PARSE, NULL);
+		return NULL;
+	}
+	if((fgets(line, sizeof(line), f)) == NULL) {
+		sg_set_error(SG_ERROR_PARSE, NULL);
+		return NULL;
+	}
 
-        while((fgets(line, sizeof(line), f)) != NULL){
-                char *name, *ptr;
-                struct ifreq ifr;
-                struct ethtool_cmd ethcmd;
-                int err;
+	while((fgets(line, sizeof(line), f)) != NULL){
+		char *name, *ptr;
+		struct ifreq ifr;
+		struct ethtool_cmd ethcmd;
+		int err;
 
 		/* Get the interface name */
-                ptr = strchr(line, ':');
-                if (ptr == NULL) continue;
-                *ptr='\0';
-                name = line;
-                while(isspace(*(name))){
-                        name++;
-                }
+		ptr = strchr(line, ':');
+		if (ptr == NULL) continue;
+		*ptr='\0';
+		name = line;
+		while(isspace(*(name))){
+			name++;
+		}
 
-                memset(&ifr, 0, sizeof ifr);
-                strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
+		memset(&ifr, 0, sizeof ifr);
+		strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);
 
 		if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0) {
 			continue;
 		}
 
 		/* We have a good interface to add */
-		network_iface_stats=network_iface_stat_malloc((ifaces+1), &sizeof_network_iface_stats, network_iface_stats);
-		if(network_iface_stats==NULL){
+		if (VECTOR_RESIZE(network_iface_stats, ifaces + 1) < 0) {
 			return NULL;
 		}
 		network_iface_stat_ptr = network_iface_stats + ifaces;
-		network_iface_stat_ptr->interface_name = strdup(name);
+		
+		if (sg_update_string(&network_iface_stat_ptr->interface_name,
+				     name) < 0) {
+			return NULL;
+		}
 		if ((ifr.ifr_flags & IFF_UP) != 0) {
 			network_iface_stat_ptr->up = 1;
 		} else {
 			network_iface_stat_ptr->up = 0;
 		}
 
-                memset(&ethcmd, 0, sizeof ethcmd);
-                ethcmd.cmd = ETHTOOL_GSET;
-                ifr.ifr_data = (caddr_t) &ethcmd;
+		memset(&ethcmd, 0, sizeof ethcmd);
+		ethcmd.cmd = ETHTOOL_GSET;
+		ifr.ifr_data = (caddr_t) &ethcmd;
 
-                err = ioctl(sock, SIOCETHTOOL, &ifr);
-                if (err == 0) {
+		err = ioctl(sock, SIOCETHTOOL, &ifr);
+		if (err == 0) {
 			network_iface_stat_ptr->speed = ethcmd.speed;
 
 			switch (ethcmd.duplex) {
 			case 0x00:
-				network_iface_stat_ptr->dup = FULL_DUPLEX;
+				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
 				break;
 			case 0x01:
-				network_iface_stat_ptr->dup = HALF_DUPLEX;
+				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
 				break;
 			default:
-				network_iface_stat_ptr->dup = UNKNOWN_DUPLEX;
+				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
 			}
 		} else {
 			/* Not all interfaces support the ethtool ioctl. */
 			network_iface_stat_ptr->speed = 0;
-			network_iface_stat_ptr->dup = UNKNOWN_DUPLEX;
+			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
 		}
 
 		ifaces++;
@@ -719,3 +695,10 @@ network_iface_stat_t *get_network_iface_stats(int *entries){
 	return network_iface_stats; 
 }
 
+int sg_network_iface_compare_name(const void *va, const void *vb) {
+	const sg_network_iface_stats *a = (const sg_network_iface_stats *)va;
+	const sg_network_iface_stats *b = (const sg_network_iface_stats *)vb;
+
+	return strcmp(a->interface_name, b->interface_name);
+}
+
