@@ -1,5 +1,5 @@
 /*
- * i-scream central monitoring system
+ * i-scream libstatgrab
  * http://www.i-scream.org
  * Copyright (C) 2000-2004 i-scream
  * 
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: os_info.c,v 1.14 2004/02/16 14:55:32 tdb Exp $
+ * $Id: os_info.c,v 1.18 2004/04/07 21:08:40 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,9 +47,9 @@
 #include <sys/time.h>
 #endif
 
-general_stat_t *get_general_stats(){
+sg_host_info *sg_get_host_info(){
 
-	static general_stat_t general_stat;	
+	static sg_host_info general_stat;	
 	static struct utsname os;
 
 #ifdef SOLARIS
@@ -69,27 +69,32 @@ general_stat_t *get_general_stats(){
 #endif
 
 	if((uname(&os)) < 0){
+		sg_set_error(SG_ERROR_UNAME, NULL);
 		return NULL;
 	}
 	
 	general_stat.os_name = os.sysname;
-        general_stat.os_release = os.release;
-        general_stat.os_version = os.version;
-        general_stat.platform = os.machine;
-        general_stat.hostname = os.nodename;
+	general_stat.os_release = os.release;
+	general_stat.os_version = os.version;
+	general_stat.platform = os.machine;
+	general_stat.hostname = os.nodename;
 
 	/* get uptime */
 #ifdef SOLARIS
 	if ((kc = kstat_open()) == NULL) {
+		sg_set_error(SG_ERROR_KSTAT_OPEN, NULL);
 		return NULL;
 	}
 	if((ksp=kstat_lookup(kc, "unix", -1, "system_misc"))==NULL){
+		sg_set_error(SG_ERROR_KSTAT_LOOKUP, "unix,-1,system_misc");
 		return NULL;
 	}
 	if (kstat_read(kc, ksp, 0) == -1) {
+		sg_set_error(SG_ERROR_KSTAT_READ, NULL);
 		return NULL;
 	}
 	if((kn=kstat_data_lookup(ksp, "boot_time")) == NULL){
+		sg_set_error(SG_ERROR_KSTAT_DATA_LOOKUP, "boot_time");
 		return NULL;
 	}
 	boottime=(kn->value.ui32);
@@ -101,9 +106,11 @@ general_stat_t *get_general_stats(){
 #endif
 #if defined(LINUX) || defined(CYGWIN)
 	if ((f=fopen("/proc/uptime", "r")) == NULL) {
+		sg_set_error(SG_ERROR_OPEN, "/proc/uptime");
 		return NULL;
 	}
 	if((fscanf(f,"%lu %*d",&general_stat.uptime)) != 1){
+		sg_set_error(SG_ERROR_PARSE, NULL);
 		return NULL;
 	}
 	fclose(f);
@@ -113,6 +120,7 @@ general_stat_t *get_general_stats(){
 	mib[1] = KERN_BOOTTIME;
 	size = sizeof boottime;
 	if (sysctl(mib, 2, &boottime, &size, NULL, 0) < 0){
+		sg_set_error(SG_ERROR_SYSCTL, "CTL_KERN.KERN_BOOTTIME");
 		return NULL;
 	}
 	time(&curtime);
