@@ -1,5 +1,5 @@
 /*
- * i-scream central monitoring system
+ * i-scream libstatgrab
  * http://www.i-scream.org
  * Copyright (C) 2000-2004 i-scream
  * 
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: page_stats.c,v 1.17 2004/02/16 14:55:32 tdb Exp $
+ * $Id: page_stats.c,v 1.21 2004/04/07 21:08:40 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,14 +46,14 @@
 #include <uvm/uvm.h>
 #endif
 
-static page_stat_t page_stats;
+static sg_page_stats page_stats;
 static int page_stats_uninit=1;
 
-page_stat_t *get_page_stats(){
+sg_page_stats *sg_get_page_stats(){
 #ifdef SOLARIS
-        kstat_ctl_t *kc;
-        kstat_t *ksp;
-        cpu_stat_t cs;
+	kstat_ctl_t *kc;
+	kstat_t *ksp;
+	cpu_stat_t cs;
 #endif
 #if defined(LINUX) || defined(CYGWIN)
 	FILE *f;
@@ -67,18 +67,19 @@ page_stat_t *get_page_stats(){
 #endif
 
 	page_stats.systime = time(NULL);
-        page_stats.pages_pagein=0;
-        page_stats.pages_pageout=0;
+	page_stats.pages_pagein=0;
+	page_stats.pages_pageout=0;
 
 #ifdef SOLARIS
-        if ((kc = kstat_open()) == NULL) {
-                return NULL;
-        }
-        for (ksp = kc->kc_chain; ksp!=NULL; ksp = ksp->ks_next) {
-                if ((strcmp(ksp->ks_module, "cpu_stat")) != 0) continue;
-                if (kstat_read(kc, ksp, &cs) == -1) {
-                        continue;
-                }
+	if ((kc = kstat_open()) == NULL) {
+		sg_set_error(SG_ERROR_KSTAT_OPEN, NULL);
+		return NULL;
+	}
+	for (ksp = kc->kc_chain; ksp!=NULL; ksp = ksp->ks_next) {
+		if ((strcmp(ksp->ks_module, "cpu_stat")) != 0) continue;
+		if (kstat_read(kc, ksp, &cs) == -1) {
+			continue;
+		}
 
 		page_stats.pages_pagein+=(long long)cs.cpu_vminfo.pgpgin;
 		page_stats.pages_pageout+=(long long)cs.cpu_vminfo.pgpgout;
@@ -88,7 +89,7 @@ page_stat_t *get_page_stats(){
 #endif
 #if defined(LINUX) || defined(CYGWIN)
 	if ((f = fopen("/proc/vmstat", "r")) != NULL) {
-		while ((line_ptr = f_read_line(f, "")) != NULL) {
+		while ((line_ptr = sg_f_read_line(f, "")) != NULL) {
 			long long value;
 
 			if (sscanf(line_ptr, "%*s %lld", &value) != 1) {
@@ -104,33 +105,38 @@ page_stat_t *get_page_stats(){
 
 		fclose(f);
 	} else if ((f = fopen("/proc/stat", "r")) != NULL) {
-		if ((line_ptr = f_read_line(f, "page")) == NULL) {
+		if ((line_ptr = sg_f_read_line(f, "page")) == NULL) {
+			sg_set_error(SG_ERROR_PARSE, "page");
 			fclose(f);
 			return NULL;
 		}
 
 		if (sscanf(line_ptr, "page %lld %lld", &page_stats.pages_pagein, &page_stats.pages_pageout) != 2) {
+			sg_set_error(SG_ERROR_PARSE, "page");
 			fclose(f);
 			return NULL;
 		}
 
 		fclose(f);
 	} else {
+		sg_set_error(SG_ERROR_OPEN, "/proc/stat");
 		return NULL;
 	}
 #endif
 #if defined(FREEBSD) || defined(DFBSD)
 	size = sizeof page_stats.pages_pagein;
-        if (sysctlbyname("vm.stats.vm.v_swappgsin", &page_stats.pages_pagein, &size, NULL, 0) < 0){
-                return NULL;
-        }
+	if (sysctlbyname("vm.stats.vm.v_swappgsin", &page_stats.pages_pagein, &size, NULL, 0) < 0){
+		sg_set_error(SG_ERROR_SYSCTLBYNAME, "vm.stats.vm.v_swappgsin");
+		return NULL;
+	}
 	size = sizeof page_stats.pages_pageout;
-        if (sysctlbyname("vm.stats.vm.v_swappgsout", &page_stats.pages_pageout, &size, NULL, 0) < 0){
-                return NULL;
-        }
+	if (sysctlbyname("vm.stats.vm.v_swappgsout", &page_stats.pages_pageout, &size, NULL, 0) < 0){
+		sg_set_error(SG_ERROR_SYSCTLBYNAME, "vm.stats.vm.v_swappgsout");
+		return NULL;
+	}
 #endif
 #if defined(NETBSD) || defined(OPENBSD)
-	if ((uvm = get_uvmexp()) == NULL) {
+	if ((uvm = sg_get_uvmexp()) == NULL) {
 		return NULL;
 	}
 
@@ -141,12 +147,12 @@ page_stat_t *get_page_stats(){
 	return &page_stats;
 }
 
-page_stat_t *get_page_stats_diff(){
-	page_stat_t *page_ptr;
-	static page_stat_t page_stats_diff;
+sg_page_stats *sg_get_page_stats_diff(){
+	sg_page_stats *page_ptr;
+	static sg_page_stats page_stats_diff;
 
 	if(page_stats_uninit){
-		page_ptr=get_page_stats();
+		page_ptr=sg_get_page_stats();
 		if(page_ptr==NULL){
 			return NULL;
 		}
@@ -158,14 +164,14 @@ page_stat_t *get_page_stats_diff(){
 	page_stats_diff.pages_pageout=page_stats.pages_pageout;
 	page_stats_diff.systime=page_stats.systime;
 
-	page_ptr=get_page_stats();
+	page_ptr=sg_get_page_stats();
 	if(page_ptr==NULL){
 		return NULL;
 	}
 
-        page_stats_diff.pages_pagein=page_stats.pages_pagein-page_stats_diff.pages_pagein;
-        page_stats_diff.pages_pageout=page_stats.pages_pageout-page_stats_diff.pages_pageout;
-        page_stats_diff.systime=page_stats.systime-page_stats_diff.systime;
+	page_stats_diff.pages_pagein=page_stats.pages_pagein-page_stats_diff.pages_pagein;
+	page_stats_diff.pages_pageout=page_stats.pages_pageout-page_stats_diff.pages_pageout;
+	page_stats_diff.systime=page_stats.systime-page_stats_diff.systime;
 	
 	return &page_stats_diff;
 }
