@@ -1,5 +1,5 @@
 /*
- * i-scream central monitoring system
+ * i-scream libstatgrab
  * http://www.i-scream.org
  * Copyright (C) 2000-2004 i-scream
  * 
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: swap_stats.c,v 1.16 2004/02/16 14:55:32 tdb Exp $
+ * $Id: swap_stats.c,v 1.20 2004/04/07 21:08:40 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,20 +42,21 @@
 #include <sys/sysctl.h>
 #include <sys/user.h>
 #else
-#include <unistd.h>
 #include <sys/types.h>
 #include <kvm.h>
 #endif
+#include <unistd.h>
 #endif
 #if defined(NETBSD) || defined(OPENBSD)
 #include <sys/param.h>
 #include <sys/time.h>
 #include <uvm/uvm.h>
+#include <unistd.h>
 #endif
 
-swap_stat_t *get_swap_stats(){
+sg_swap_stats *sg_get_swap_stats(){
 
-	static swap_stat_t swap_stat;
+	static sg_swap_stats swap_stat;
 
 #ifdef SOLARIS
 	struct anoninfo ai;
@@ -83,9 +84,11 @@ swap_stat_t *get_swap_stats(){
 
 #ifdef SOLARIS
 	if((pagesize=sysconf(_SC_PAGESIZE)) == -1){
+		sg_set_error(SG_ERROR_SYSCONF, "_SC_PAGESIZE");
 		return NULL;
 	}
 	if (swapctl(SC_AINFO, &ai) == -1) {
+		sg_set_error(SG_ERROR_SWAPCTL, NULL);
 		return NULL;
 	}
 	swap_stat.total = (long long)ai.ani_max * (long long)pagesize;
@@ -94,10 +97,11 @@ swap_stat_t *get_swap_stats(){
 #endif
 #if defined(LINUX) || defined(CYGWIN)
 	if ((f = fopen("/proc/meminfo", "r")) == NULL) {
+		sg_set_error(SG_ERROR_OPEN, "/proc/meminfo");
 		return NULL;
 	}
 
-	while ((line_ptr = f_read_line(f, "")) != NULL) {
+	while ((line_ptr = sg_f_read_line(f, "")) != NULL) {
 		if (sscanf(line_ptr, "%*s %llu kB", &value) != 1) {
 			continue;
 		}
@@ -122,6 +126,7 @@ swap_stat_t *get_swap_stats(){
 
 	mibsize = sizeof mib / sizeof mib[0];
 	if (sysctlnametomib("vm.swap_info", mib, &mibsize) < 0) {
+		sg_set_error(SG_ERROR_SYSCTLNAMETOMIB, "vm.swap_info");
 		return NULL;
 	}
 	for (n = 0; ; ++n) {
@@ -131,19 +136,22 @@ swap_stat_t *get_swap_stats(){
 			break;
 		}
 		if (xsw.xsw_version != XSWDEV_VERSION) {
+			sg_set_error(SG_ERROR_XSW_VER_MISMATCH, NULL);
 			return NULL;
 		}
 		swap_stat.total += (long long) xsw.xsw_nblks;
 		swap_stat.used += (long long) xsw.xsw_used;
 	}
 	if (errno != ENOENT) {
+		sg_set_error(SG_ERROR_ENOENT, NULL);
 		return NULL;
 	}
 #else
-	if((kvmd = get_kvm()) == NULL){
+	if((kvmd = sg_get_kvm()) == NULL){
 		return NULL;
 	}
 	if ((kvm_getswapinfo(kvmd, &swapinfo, 1,0)) == -1){
+		sg_set_error(SG_ERROR_KVM_GETSWAPINFO, NULL);
 		return NULL;
 	}
 
@@ -155,7 +163,7 @@ swap_stat_t *get_swap_stats(){
 	swap_stat.free = swap_stat.total - swap_stat.used;
 #endif
 #if defined(NETBSD) || defined(OPENBSD)
-	if ((uvm = get_uvmexp()) == NULL) {
+	if ((uvm = sg_get_uvmexp()) == NULL) {
 		return NULL;
 	}
 
